# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    # https://www.youtube.com/watch?v=dYnDCHUzzaY
    # ":" is a possible separator, but should not be used in this case.
    {
        "input": 'HA:TFELT [핫펠트(예은)] "Truth" M/V',
        # Ideal would be to include the Hangul but it's in
        # [] which means it's deleted for now.
        "expected": ["HA:TFELT", "Truth"],
        "skip": True,
    },
    # https://www.youtube.com/watch?v=Qk52ypnGs68
    # "-" is a possible separator, but should not be used in this case.
    {
        "input": 'T-ARA[티아라] "NUMBER NINE [넘버나인]" M/V',
        "expected": ["T-ARA", "NUMBER NINE"],
        "skip": True,
    },
    # https://www.youtube.com/watch?v=aeo_nWsu5cs
    {
        "input": "[MV] YOUNHA(윤하) _ Get It?(알아듣겠지) (Feat. HA:TFELT, CHEETAH(치타))",
        "expected": ["YOUNHA(윤하)", "Get It?(알아듣겠지) (Feat. HA:TFELT, CHEETAH(치타))"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Base method to parse title
"""
import re
from functools import reduce

SEPARATORS = [
    " -- ",
    "--",
    " - ",
    " – ",
    " — ",
    " _ ",
    "-",
    "–",
    "—",
    ":",
    "|",
    "///",
    " / ",
    "_",
    "/",
]


def clean_mvpv(text):
    """
    Remove various versions of "MV" and "PV" markers
    """
    text = re.sub(
        r"\s*\[\s*(?:off?icial\s+)?([PM]\/?V)\s*]", "", text, flags=re.IGNORECASE
    )  # [MV] or [M/V]
    text = re.sub(
        r"\s*\(\s*(?:off?icial\s+)?([PM]\/?V)\s*\)", "", text, flags=re.IGNORECASE
    )  # (MV) or (M/V)
    text = re.sub(
        r"\s*【\s*(?:off?icial\s+)?([PM]\/?V)\s*】", "", text, flags=re.IGNORECASE
    )  # 【MV】 or 【M/V】
    text = re.sub(
        r"[\s\-–_]+(?:off?icial\s+)?([PM]\/?V)\s*", "", text, flags=re.IGNORECASE
    )  # MV or M/V at the end
    text = re.sub(
        r"(?:off?icial\s+)?([PM]\/?V)[\s\-–_]+", "", text
    )  # MV or M/V at the start
    return text


def clean_fluff(text):
    """
    Remove fluff
    """
    text = clean_mvpv(text)
    text = re.sub(r"\s*\[[^\]]+]$", "", text)  # [whatever] at the end
    text = re.sub(r"^\s*\[[^\]]+]\s*", "", text)  # [whatever] at the start
    text = re.sub(
        r"\s*\([^)]*\bver(\.|sion)?\s*\)$", "", text, flags=re.IGNORECASE
    )  # (whatever version)
    text = re.sub(
        r"\s*[a-z]*\s*\bver(\.|sion)?$", "", text, flags=re.IGNORECASE
    )  # ver. & 1 word before (no parens)
    text = re.sub(
        r"\s*(of+icial\s*)?(music\s*)?video", "", text, flags=re.IGNORECASE
    )  # (official)? (music)? video
    text = re.sub(r"\s*(full\s*)?album", "", text, flags=re.IGNORECASE)  # (full)? album
    text = re.sub(
        r"\s*(ALBUM TRACK\s*)?(album track\s*)", "", text, flags=re.IGNORECASE
    )  # (ALBUM TRACK)
    text = re.sub(r"\s*\(\s*of+icial\s*\)", "", text, flags=re.IGNORECASE)  # (official)
    text = re.sub(r"\s*\(\s*lyric(s)?\s*\)", "", text, flags=re.IGNORECASE)  # (lyrics)
    text = re.sub(
        r"\s*\(\s*(of+icial)?\s*lyric(s)?\s*\)", "", text, flags=re.IGNORECASE
    )  # (official lyrics)
    text = re.sub(r"\s*\(\s*[0-9]{4}\s*\)", "", text, flags=re.IGNORECASE)  # (1999)
    text = re.sub(
        r"\s+\(\s*(HD|HQ|[0-9]{3,4}p|4K)\s*\)$", "", text
    )  # (HD) (HQ) (1080p) (4K)
    text = re.sub(
        r"[\s\-–_]+(HD|HQ|[0-9]{3,4}p|4K)\s*$", "", text
    )  # - HD - HQ - 720p - 4K
    return text


def clean_title(title):
    """
    Clean song title
    """
    title = title.strip(" ")
    title = clean_fluff(title)
    title = re.sub(r"\s*\*+\s?\S+\s?\*+$", "", title)  # **NEW**
    title = re.sub(r"\s*video\s*clip", "", title, flags=re.IGNORECASE)  # video clip
    title = re.sub(r"\s+\(?live\)?$", "", title, flags=re.IGNORECASE)  # live
    title = re.sub(r"\(\s*\)", "", title)  # Leftovers after e.g. (official video)
    title = re.sub(r"\[\s*]", "", title)  # Leftovers after e.g. [1080p]
    title = re.sub(r"【\s*】", "", title)  # Leftovers after e.g. 【MV】
    title = re.sub(
        r"^(|.*\s)\"(.*)\"(\s.*|)$", r"\2", title
    )  # Artist - The new "Track title" featuring someone
    title = re.sub(r"^(|.*\s)'(.*)'(\s.*|)$", r"\2", title)  # 'Track title'
    title = re.sub(
        r"^[/\s,:;~\-–_\s\"]+", "", title
    )  # trim starting white chars and dash
    title = re.sub(
        r"[/\s,:;~\-–_\s\"]+$", "", title
    )  # trim trailing white chars and dash
    return title


def clean_artist(artist):
    """
    Clean artist name
    """
    artist = artist.strip(" ")
    artist = clean_fluff(artist)
    artist = re.sub(
        r"\s*[0-1][0-9][0-1][0-9][0-3][0-9]\s*", "", artist
    )  # date formats ex. 130624
    artist = re.sub(
        r"^[/\s,:;~\-–_\s\"]+", "", artist
    )  # trim starting white chars and dash
    artist = re.sub(
        r"[/\s,:;~\-–_\s\"]+$", "", artist
    )  # trim trailing white chars and dash
    return artist


def in_quotes(text, idx):
    open_chars = "([{«"
    close_chars = ")]}»"
    toggle_chars = "\"'"
    open_pars = {")": 0, "]": 0, "}": 0, "»": 0, '"': 0, "'": 0}
    for character in text[:idx]:
        index = open_chars.find(character)
        if index != -1:
            open_pars[close_chars[index]] += 1
        elif close_chars.find(character) != -1 and open_pars[character] > 0:
            open_pars[character] -= 1
        if toggle_chars.find(character) != -1:
            open_pars[character] = 1 - open_pars[character]
    return reduce((lambda acc, value: acc + value), open_pars.values()) > 0


def split_artist_title(text):
    """
    Split text at separators
    """
    for separator in SEPARATORS:
        try:
            idx = text.index(separator)
        except ValueError:
            continue
        if idx > -1 and not in_quotes(text, idx):
            return [text[:idx], text[idx + len(separator) :]]



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    {
        "input": "Artist",
        "expected": ["Artist", "Title"],
        "options": {"defaultTitle": "Title"},
    }
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Fallback to given title when no title/artist detected
"""


def fallback_title(title):
    """
    Fallback method
    """

    def fallback_t(artist):
        return [artist, title]

    return fallback_t



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    {
        "input": "clipping. - Inside Out [OFFICIAL VIDEO]",
        "expected": ["clipping.", "Inside Out"],
    },
    {
        "input": "King Tuff - Black Moon Spell [OFFICIAL VIDEO]",
        "expected": ["King Tuff", "Black Moon Spell"],
    },
    {
        "input": "Sleater-Kinney - Bury Our Friends (feat. Miranda July)",
        "expected": ["Sleater-Kinney", "Bury Our Friends (feat. Miranda July)"],
    },
    {
        "input": "Pissed Jeans - Boring Girls [OFFICIAL VIDEO]",
        "expected": ["Pissed Jeans", "Boring Girls"],
    },
    {
        "input": "Father John Misty - Chateau Lobby #4 (in C for Two Virgins) [OFFICIAL VIDEO]",
        "expected": ["Father John Misty", "Chateau Lobby #4 (in C for Two Virgins)"],
    },
    {
        "input": "Chad VanGaalen - Monster [OFFICIAL VIDEO]",
        "expected": ["Chad VanGaalen", "Monster"],
    },
    {
        "input": "Luluc - Tangled Heart  [OFFICIAL VIDEO]",
        "expected": ["Luluc", "Tangled Heart"],
    },
    {
        "input": "J Mascis - Every Morning [OFFICIAL VIDEO]",
        "expected": ["J Mascis", "Every Morning"],
    },
    {
        "input": "THEESatisfaction - Recognition  (not the video)",
        "expected": ["THEESatisfaction", "Recognition"],
    },
    {
        "input": "Goat - Hide from the Sun [OFFICIAL VIDEO]",
        "expected": ["Goat", "Hide from the Sun"],
    },
    {
        "input": "Shabazz Palaces - Motion Sickness [OFFICIAL VIDEO]",
        "expected": ["Shabazz Palaces", "Motion Sickness"],
    },
    {
        "input": "The Afghan Whigs - Lost in the Woods [OFFICIAL VIDEO]",
        "expected": ["The Afghan Whigs", "Lost in the Woods"],
    },
    {
        "input": "The Head and the Heart - Another Story [OFFICIAL VIDEO]",
        "expected": ["The Head and the Heart", "Another Story"],
    },
    {
        "input": "Washed Out - Weightless [OFFICIAL VIDEO]",
        "expected": ["Washed Out", "Weightless"],
    },
    {
        "input": "Deaf Wish - Cool Comment [OFFICIAL VIDEO]",
        "expected": ["Deaf Wish", "Cool Comment"],
    },
    {
        "input": "Father John Misty - Bored In The USA",
        "expected": ["Father John Misty", "Bored In The USA"],
    },
    {
        "input": "Flake Music - Spanway Hits  (not the video)",
        "expected": ["Flake Music", "Spanway Hits"],
    },
    {
        "input": "Sleater-Kinney - Surface Envy",
        "expected": ["Sleater-Kinney", "Surface Envy"],
    },
    {
        "input": "clipping. - Get Up [OFFICIAL VIDEO]",
        "expected": ["clipping.", "Get Up"],
    },
    {"input": "King Tuff - Headbanger", "expected": ["King Tuff", "Headbanger"]},
    {
        "input": "Shabazz Palaces - #CAKE [OFFICIAL VIDEO]",
        "expected": ["Shabazz Palaces", "#CAKE"],
    },
    {
        "input": "The Afghan Whigs - Every Little Thing She Does is Magic (cover of The Police)",
        "expected": [
            "The Afghan Whigs",
            "Every Little Thing She Does is Magic (cover of The Police)",
        ],
    },
    {
        "input": "Mirel Wagner - The Dirt [OFFICIAL VIDEO]",
        "expected": ["Mirel Wagner", "The Dirt"],
    },
    {
        "input": "Avi Buffalo - Memories of You (not the video)",
        "expected": ["Avi Buffalo", "Memories of You"],
    },
    {"input": "Goat - Words (not the video)", "expected": ["Goat", "Words"]},
    {
        "input": "clipping. - Story 2 [OFFICIAL VIDEO]",
        "expected": ["clipping.", "Story 2"],
    },
    {
        "input": "Shabazz Palaces - #CAKE (Animal Collective Premature Deflirt Mix)",
        "expected": [
            "Shabazz Palaces",
            "#CAKE (Animal Collective Premature Deflirt Mix)",
        ],
    },
    {"input": "Flake Music - The Shins", "expected": ["Flake Music", "The Shins"]},
    {
        "input": "J Mascis - Wide Awake (not the video)",
        "expected": ["J Mascis", "Wide Awake"],
    },
    {
        "input": "The Head and the Heart - Let's Be Still [OFFICIAL VIDEO]",
        "expected": ["The Head and the Heart", "Let's Be Still"],
    },
    {
        "input": "Luluc - Small Window [OFFICIAL VIDEO]",
        "expected": ["Luluc", "Small Window"],
    },
    {
        "input": "Lyla Foy - Honeymoon [OFFICIAL VIDEO]",
        "expected": ["Lyla Foy", "Honeymoon"],
    },
    {
        "input": "Avi Buffalo - So What [OFFICIAL VIDEO]",
        "expected": ["Avi Buffalo", "So What"],
    },
    {
        "input": "clipping. - Body & Blood [CENSORED VERSION of OFFICIAL VIDEO]",
        "expected": ["clipping.", "Body & Blood"],
    },
    {
        "input": "King Tuff - Eyes of the Muse (not the video)",
        "expected": ["King Tuff", "Eyes of the Muse"],
    },
    {"input": "Deaf Wish - St Vincent's", "expected": ["Deaf Wish", "St Vincent's"]},
    {
        "input": "The Afghan Whigs - Matamoros [OFFICIAL VIDEO]",
        "expected": ["The Afghan Whigs", "Matamoros"],
    },
    {
        "input": "Dum Dum Girls - Too True To Be Good [OFFICIAL VIDEO]",
        "expected": ["Dum Dum Girls", "Too True To Be Good"],
    },
    {
        "input": "J Mascis - Fade Into You (a Mazzy Star cover)",
        "expected": ["J Mascis", "Fade Into You (a Mazzy Star cover)"],
    },
    {
        "input": "Shabazz Palaces - Forerunner Foray",
        "expected": ["Shabazz Palaces", "Forerunner Foray"],
    },
    {
        "input": "Lee Bains III & The Glory Fires - The Company Man [OFFICIAL VIDEO]",
        "expected": ["Lee Bains III & The Glory Fires", "The Company Man"],
    },
    {
        "input": "The Postal Service - Nothing Better [LIVE]",
        "expected": ["The Postal Service", "Nothing Better"],
    },
    {
        "input": "Rose Windows - There is a Light [OFFICIAL VIDEO]",
        "expected": ["Rose Windows", "There is a Light"],
    },
    {
        "input": "Mirel Wagner - Oak Tree [OFFICIAL VIDEO]",
        "expected": ["Mirel Wagner", "Oak Tree"],
    },
    {
        "input": "clipping. - Work Work (feat. Cocc Pistol Cree) [OFFICIAL VIDEO]",
        "expected": ["clipping.", "Work Work (feat. Cocc Pistol Cree)"],
    },
    {
        "input": "Dum Dum Girls - Rimbaud Eyes [OFFICIAL VIDEO]",
        "expected": ["Dum Dum Girls", "Rimbaud Eyes"],
    },
    {
        "input": "The Head and the Heart - Summertime [OFFICIAL VIDEO]",
        "expected": ["The Head and the Heart", "Summertime"],
    },
    {
        "input": "Shabazz Palaces - They Come In Gold (not the video)",
        "expected": ["Shabazz Palaces", "They Come In Gold"],
    },
    {
        "input": "Lee Bains III & The Glory Fires - The Weeds Downtown (not the video)",
        "expected": ["Lee Bains III & The Glory Fires", "The Weeds Downtown"],
    },
    {
        "input": "Constantines - Young Lions (not the video)",
        "expected": ["Constantines", "Young Lions"],
    },
    {
        "input": "The Afghan Whigs - The Lottery (not the video)",
        "expected": ["The Afghan Whigs", "The Lottery"],
    },
    {"input": "Luluc - Without a Face", "expected": ["Luluc", "Without a Face"]},
    {
        "input": "Lyla Foy - Feather Tongue [OFFICIAL VIDEO]",
        "expected": ["Lyla Foy", "Feather Tongue"],
    },
    {
        "input": "Dum Dum Girls -  Are You Okay? [OFFICIAL SHORT FILM VIDEO]",
        "expected": ["Dum Dum Girls", "Are You Okay?"],
    },
    {
        "input": "THUMPERS - Lungs (Originally Performed by Chvrches) [not the video]",
        "expected": ["THUMPERS", "Lungs (Originally Performed by Chvrches)"],
    },
    {
        "input": "Washed Out - All I Know [Moby Remix] (not the video)",
        "expected": ["Washed Out", "All I Know [Moby Remix]"],
    },
    {
        "input": "Goat - Dreambuilding (not the video)",
        "expected": ["Goat", "Dreambuilding"],
    },
    {
        "input": "Lee Bains III & The Glory Fires - The Company Man (not the video)",
        "expected": ["Lee Bains III & The Glory Fires", "The Company Man"],
    },
    {
        "input": "The Afghan Whigs - Algiers [OFFICIAL VIDEO]",
        "expected": ["The Afghan Whigs", "Algiers"],
    },
    {
        "input": 'Shearwater - Black Is The Color  (from the "Colors" episode of Radiolab)',
        "expected": [
            "Shearwater",
            'Black Is The Color (from the "Colors" episode of Radiolab)',
        ],
        "skip": True,
    },
    {
        "input": "The Notwist - Kong [OFFICIAL VIDEO]",
        "expected": ["The Notwist", "Kong"],
    },
    {
        "input": "Death Vessel - Mercury Dime [OFFICIAL VIDEO]",
        "expected": ["Death Vessel", "Mercury Dime"],
    },
    {
        "input": "The Ruby Suns - Desert Of Pop [OFFICIAL VIDEO]",
        "expected": ["The Ruby Suns", "Desert Of Pop"],
    },
    {
        "input": "THUMPERS - Unkinder (A Tougher Love) [OFFICIAL VIDEO]",
        "expected": ["THUMPERS", "Unkinder (A Tougher Love)"],
    },
    {
        "input": "Chad VanGaalen - Where Are You?",
        "expected": ["Chad VanGaalen", "Where Are You?"],
    },
    {
        "input": "Mogwai - The Lord Is Out Of Control [OFFICIAL VIDEO]",
        "expected": ["Mogwai", "The Lord Is Out Of Control"],
    },
    {
        "input": "Lyla Foy -  Impossible [OFFICIAL PERFORMANCE VIDEO]",
        "expected": ["Lyla Foy", "Impossible"],
    },
    {
        "input": "Death Vessel - Ilsa Drown (feat. Jónsi) [not the video]",
        "expected": ["Death Vessel", "Ilsa Drown (feat. Jónsi)"],
    },
    {
        "input": "The Notwist - Close To The Glass (not the video)",
        "expected": ["The Notwist", "Close To The Glass"],
    },
    {
        "input": "Dum Dum Girls - Lost Boys And Girls Club [OFFICIAL VIDEO]",
        "expected": ["Dum Dum Girls", "Lost Boys And Girls Club"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Parse the title of a YouTube video to try and get artist & song name
"""
from __future__ import print_function, absolute_import
import argparse

try:
    import youtube_title_parse.plugins as plugins
    from youtube_title_parse.core import mapArtistTitle, mapTitle, get_song_artist_title
except ImportError:
    import plugins
    from core import mapArtistTitle, mapTitle, get_song_artist_title


def get_artist_title(text, options={}):
    """
    Parse method
    """
    result = get_song_artist_title(
        text,
        options,
        {
            "before": [plugins.remove_file_extensions, plugins.clean_fluff],
            "split": [plugins.split_artist_title, plugins.split_text],
            "after": [
                mapArtistTitle(plugins.clean_artist, plugins.clean_title),
                mapArtistTitle(plugins.clean, plugins.clean),
                mapTitle(plugins.clean_common_fluff),
            ],
        },
    )
    if result:
        return result[0], result[1]
    else:
        return None


def process(args):
    options = {}
    if args.defaultArtist:
        options["defaultArtist"] = args.defaultArtist
    if args.defaultTitle:
        options["defaultTitle"] = args.defaultTitle
    result = get_artist_title(args.youtube_title, options)
    if result:
        print(result)
    else:
        print(args.youtube_title)
    return result


def main():
    argparser = argparse.ArgumentParser(
        description="youtube_title_parse", formatter_class=argparse.RawTextHelpFormatter
    )
    argparser.add_argument(
        "youtube_title", type=str, help="required youtube video title"
    )
    argparser.add_argument(
        "-da",
        "--defaultArtist",
        dest="defaultArtist",
        type=str,
        metavar="DEFAULT_ARTIST",
        default=False,
        help="""
        fallback artist name
        This value will be used as the 'artist' if
        parser couldn't get artist from YouTube title.
        """,
    )
    argparser.add_argument(
        "-dt",
        "--defaultTitle",
        dest="defaultTitle",
        type=str,
        metavar="DEFAULT_TITLE",
        default=False,
        help="""
        fallback song title
        This value will be used as the 'title' if
        parser couldn't get song title from YouTube title.
        """,
    )
    args = argparser.parse_args()
    process(args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Remove file extensions from string
"""
import re

VIDEO_EXTENSIONS = [
    "3g2",
    "3gp",
    "aaf",
    "asf",
    "avchd",
    "avi",
    "drc",
    "flv",
    "m2v",
    "m4p",
    "m4v",
    "mkv",
    "mng",
    "mov",
    "mp2",
    "mp4",
    "mpe",
    "mpeg",
    "mpg",
    "mpv",
    "mxf",
    "nsv",
    "ogg",
    "ogv",
    "qt",
    "rm",
    "rmvb",
    "roq",
    "svi",
    "vob",
    "webm",
    "wmv",
    "yuv",
]

AUDIO_EXTENSIONS = [
    "wav",
    "bwf",
    "raw",
    "aiff",
    "flac",
    "m4a",
    "pac",
    "tta",
    "wv",
    "ast",
    "aac",
    "mp2",
    "mp3",
    "mp4",
    "amr",
    "s3m",
    "3gp",
    "act",
    "au",
    "dct",
    "dss",
    "gsm",
    "m4p",
    "mmf",
    "mpc",
    "ogg",
    "oga",
    "opus",
    "ra",
    "sln",
    "vox",
]

FILE_EXTENSIONS = VIDEO_EXTENSIONS + AUDIO_EXTENSIONS
FILE_EXTENSIONS_RX = re.compile(
    r"\.(" + "|".join(FILE_EXTENSIONS) + ")$", re.IGNORECASE
)


def remove_file_extensions(title):
    return FILE_EXTENSIONS_RX.sub("", title)



# -*- coding: utf-8 -*-
import os
from youtube_title_parse import get_artist_title


class MetaTestSequence(type):
    def __new__(mcs, name, bases, attrs):
        def should_skip(test_params):
            if not test_params:
                return False
            if "skip" in test_params and test_params["skip"] is True:
                return True
            return False

        def gen_test(test_name, test_params):
            input = test_params["input"]
            expected = test_params["expected"]
            skip = should_skip(test_params)

            def test_func(self):
                if skip:
                    self.skipTest("Currently unsupported")
                if "options" in test_params:
                    artist, title = get_artist_title(
                        input, options=test_params["options"]
                    ) or (None, None)
                else:
                    artist, title = get_artist_title(input) or (None, None)
                self.assertEqual(title, expected[1])
                self.assertEqual(artist, expected[0])

            test_func.__name__ = test_name
            return test_func

        if "test_cases" in attrs and "test_type" in attrs:
            for idx, test_params in enumerate(attrs["test_cases"]):
                test_kind = os.path.splitext(os.path.basename(attrs["test_type"]))[0]
                test_name = "%s_%d" % (test_kind, idx + 1)
                new_test = gen_test(test_name, test_params)
                attrs[new_test.__name__] = new_test
        return type.__new__(mcs, name, bases, attrs)



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    {
        "input": "Rush – Moving Pictures (Full Album)",
        "expected": ["Rush", "Moving Pictures"],
    },
    {
        "input": "Rush - Moving Pictures (album)",
        "expected": ["Rush", "Moving Pictures"],
    },
    {
        "input": "Rush - Moving Pictures (Official Album)",
        "expected": ["Rush", "Moving Pictures"],
    },
    {
        "input": "Rush - Moving Pictures (Full Album) (Official)",
        "expected": ["Rush", "Moving Pictures"],
    },
    {
        "input": "FILMMAKER - ETERNAL RETURN [FULL ALBUM]",
        "expected": ["FILMMAKER", "ETERNAL RETURN"],
    },
    {
        "input": "Dua Lipa - New Rules (Official Music Video) **NEW**",
        "expected": ["Dua Lipa", "New Rules"],
    },
    {
        "input": "Muse — The 2nd Law (Full Album) [HD]",
        "expected": ["Muse", "The 2nd Law"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Core parsing methods
"""
try:
    from youtube_title_parse.fallback_artist import fallback_artist
    from youtube_title_parse.fallback_title import fallback_title
except ImportError:
    from fallback_artist import fallback_artist
    from fallback_title import fallback_title


def flow(functions):
    if not functions:

        def failed(arg):
            return arg

        return failed

    def flow_func(arguments):
        result = functions[0](arguments)
        for function in functions[1:]:
            result = function(result)
        return result

    return flow_func


def combine_splitters(splitters):
    def combine_func(text):
        for splitter in splitters:
            result = splitter(text)
            if result:
                return result

    return combine_func


def reduce_plugins(plugins):
    return [
        flow(plugins["before"]),
        combine_splitters(plugins["split"]),
        flow(plugins["after"]),
    ]


def checkPlugin(plugin):
    if not plugin[1]:
        print("no title splitter was specified by any plugin")


def mapArtist(fn):
    def mapA(parts):
        return [fn(parts[0]), parts[1]]

    return mapA


def mapTitle(fn):
    def mapT(parts):
        return [parts[0], fn(parts[1])]

    return mapT


def mapArtistTitle(map_artist, map_title):
    def mapAT(parts):
        return [map_artist(parts[0]), map_title(parts[1])]

    return mapAT


def get_song_artist_title(text, options, plugins):
    if options:
        if "defaultArtist" in options:
            plugins["split"].append(fallback_artist(options["defaultArtist"]))
        if "defaultTitle" in options:
            plugins["split"].append(fallback_title(options["defaultTitle"]))
    plugin = reduce_plugins(plugins)
    checkPlugin(plugin)

    split = plugin[1](plugin[0](text))
    if not split:
        return
    return plugin[2](split)



# -*- coding: utf-8 -*-
"""
Fallback to given artist name when no title/artist detected
"""


def fallback_artist(artist):
    """
    Fallback method
    """

    def fallback_a(title):
        return [artist, title]

    return fallback_a



# -*- coding: utf-8 -*-
"""
Handle titles with quotes
"""
import re
from functools import reduce

QUOTES = ["“”", ("\x9c", "\xe2\x80\x9d"), '""', "''"]


def looseRegs(sset):
    open_set = sset[0]
    close_set = sset[1]
    return re.compile(open_set + r"(.*?)" + close_set)


def startRegs(sset):
    open_set = sset[0]
    close_set = sset[1]
    return re.compile(r"^" + open_set + r"(.*?)" + close_set + r"\s*")


def split_text(text):
    for loose_rex in map(looseRegs, QUOTES):
        text = re.sub(loose_rex, lambda re_match: r" %s " % re_match.group(), text, 1)
        match = re.search(loose_rex, text)
        if match:
            split = match.start()
            title = text[split:]
            artist = text[:split]
            return [artist, title]


def clean(artistOrTitle):
    return reduce(
        (lambda text, rx: re.sub(rx, r"\1 ", text)),
        map(startRegs, QUOTES),
        artistOrTitle,
    ).strip()



# -*- coding: utf-8 -*-
from .base import clean_fluff, split_artist_title, clean_artist, clean_title
from .remove_file_extensions import remove_file_extensions
from .quoted_title import split_text, clean
from .common import clean_common_fluff



"""
Setup script for youtube_title_parse
"""
import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

tests_require = ["six"]

setup(
    name="youtube_title_parse",
    version="1.0.0",
    description="Parse the title of a YouTube video to try and get artist & song name",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/lttkgp/youtube-title-parse",
    author="lttkgp",
    author_email="ghostwriternr@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
    keywords=["youtube", "title", "metadata", "parse", "music"],
    packages=find_packages(exclude=["contrib", "docs", "test*"]),
    install_requires=[],
    tests_require=tests_require,
    entry_points={
        "console_scripts": ["youtube_title_parse=youtube_title_parse.parse:main"],
    },
)



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    # https://youtu.be/A2RwHnfI2y8
    {
        "input": "Ga-In (가인) - Nostalgia (노스텔지아) - Lyrics [Hangul+Translation] .mov",
        "expected": ["Ga-In (가인)", "Nostalgia (노스텔지아)"],
    },
    # https://www.youtube.com/watch?v=PYBuIwuD1DA
    {"input": "show me - B-free.m4v", "expected": ["show me", "B-free"]},
    # https://www.youtube.com/watch?v=5hINYNZslP0
    {
        "input": "성시경 Sung Si Kyung - 내게 오는 길.mp4",
        "expected": ["성시경 Sung Si Kyung", "내게 오는 길"],
    },
    # Things that are NOT file extensions are not removed:
    # https://www.youtube.com/watch?v=E2yLg9iW1_0
    {"input": "에이핑크 - Mr.chu", "expected": ["에이핑크", "Mr.chu"]},
    # https://www.youtube.com/watch?v=P1Oya1PqKFc
    {
        "input": "Far East Movement - Live My Life (Feat. Justin Bieber) cover by J.Fla",
        "expected": [
            "Far East Movement",
            "Live My Life (Feat. Justin Bieber) cover by J.Fla",
        ],
    },
    # https://www.youtube.com/watch?v=rnQBF2CIygg
    # Thing that ends in a file extension without a preceding `.`:
    {
        "input": "Baka Oppai - A Piece Of Toast",
        "expected": ["Baka Oppai", "A Piece Of Toast"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

# K-pop song titles taken from JYP entertainment's Most Popular page:
# https://www.youtube.com/user/jypentertainment/videos?flow=grid&view=0&sort=p
# jypentertainment uses a title format like:
# Artist "Title" Fluff
tests = [
    # This one is not so fun because there is a separator (-) inside the quotes,
    # and separators are usually tried first.
    {
        "input": 'TWICE(트와이스) "OOH-AHH하게(Like OOH-AHH)" M/V',
        "expected": ["TWICE(트와이스)", "OOH-AHH하게(Like OOH-AHH)"],
    },
    {"input": 'GOT7 "Just right(딱 좋아)" M/V', "expected": ["GOT7", "Just right(딱 좋아)"]},
    {
        "input": "miss A “Only You(다른 남자 말고 너)” M/V",
        "expected": ["miss A", "Only You(다른 남자 말고 너)"],
    },
    {"input": 'GOT7 "If You Do(니가 하면)" M/V', "expected": ["GOT7", "If You Do(니가 하면)"]},
    {"input": 'GOT7 "A" M/V', "expected": ["GOT7", "A"]},
    {
        "input": 'J.Y. Park(박진영) "Who\'s your mama?(어머님이 누구니) (feat. Jessi)" M/V',
        "expected": ["J.Y. Park(박진영)", "Who's your mama?(어머님이 누구니) (feat. Jessi)"],
    },
    {
        "input": 'GOT7 "Girls Girls Girls" M/V',
        "expected": ["GOT7", "Girls Girls Girls"],
    },
    {
        "input": "GOT7 “Stop stop it(하지하지마)” M/V",
        "expected": ["GOT7", "Stop stop it(하지하지마)"],
    },
    {
        "input": "2PM “GO CRAZY!(미친거 아니야?)” M/V",
        "expected": ["2PM", "GO CRAZY!(미친거 아니야?)"],
    },
    {
        "input": '2PM "A.D.T.O.Y.(하.니.뿐.)" M/V',
        "expected": ["2PM", "A.D.T.O.Y.(하.니.뿐.)"],
    },
    {"input": "2PM “My House(우리집)” M/V", "expected": ["2PM", "My House(우리집)"]},
    {"input": "GOT7 “Fly” M/V", "expected": ["GOT7", "Fly"]},
    {
        "input": 'Wonder Girls "I Feel You" M/V',
        "expected": ["Wonder Girls", "I Feel You"],
    },
    {
        "input": 'GOT7 "I Like You(난 니가 좋아)" Dance Practice',
        "expected": ["GOT7", "I Like You(난 니가 좋아)"],
    },
    {
        "input": 'GOT7 "Just right(딱 좋아)" Dance Practice #2 (Just Crazy Boyfriend Ver.)',
        "expected": ["GOT7", "Just right(딱 좋아)"],
    },
    {
        "input": 'GOT7 "Stop stop it(하지하지마)" Dance Practice',
        "expected": ["GOT7", "Stop stop it(하지하지마)"],
    },
    {
        "input": '2PM "Comeback When You Hear This Song(이 노래를 듣고 돌아와)" M/V',
        "expected": ["2PM", "Comeback When You Hear This Song(이 노래를 듣고 돌아와)"],
    },
    {
        "input": 'Sunmi(선미) "Full Moon(보름달)" M/V',
        "expected": ["Sunmi(선미)", "Full Moon(보름달)"],
    },
    {
        "input": 'GOT7 "Magnetic(너란 걸)" Dance Practice',
        "expected": ["GOT7", "Magnetic(너란 걸)"],
    },
    {
        "input": 'miss A "Only You(다른 남자 말고 너)" Dance Practice',
        "expected": ["miss A", "Only You(다른 남자 말고 너)"],
    },
    {
        "input": "Baek A Yeon(백아연) “Shouldn’t Have…(이럴거면 그러지말지) (Feat. Young K)” M/V",
        "expected": ["Baek A Yeon(백아연)", "Shouldn’t Have…(이럴거면 그러지말지) (Feat. Young K)"],
    },
    {"input": 'DAY6 "Congratulations" M/V', "expected": ["DAY6", "Congratulations"]},
    {"input": 'Wonder Girls "Tell me" M/V', "expected": ["Wonder Girls", "Tell me"]},
    {
        "input": 'GOT7 "Confession Song(고백송)" M/V',
        "expected": ["GOT7", "Confession Song(고백송)"],
    },
    {
        "input": 'GOT7 "Stop stop it(하지하지마)" Dance Practice #2 (Crazy Boyfriend Ver.)',
        "expected": ["GOT7", "Stop stop it(하지하지마)"],
    },
    {"input": 'GOT7 "A" Dance Practice', "expected": ["GOT7", "A"]},
    {
        "input": 'GOT7 "Girls Girls Girls" Dance Practice #2',
        "expected": ["GOT7", "Girls Girls Girls"],
    },
    {
        "input": 'GOT7 "If You Do(니가 하면)" Dance Practice',
        "expected": ["GOT7", "If You Do(니가 하면)"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from six import with_metaclass
from .meta import MetaTestSequence

tests = [
    # https://youtu.be/hn4EIv1-uz0
    {
        "input": "Boats & BIrds - Gregory & the Hawk",
        "expected": ["Boats & BIrds", "Gregory & the Hawk"],
    },
    # https://youtu.be/JoC3PUBmhFs
    {
        "input": "Sum 41 - In Too Deep (Official Video)",
        "expected": ["Sum 41", "In Too Deep"],
    },
    # Punctuation mark at the end
    # https://youtu.be/fz3jLeDvpu4
    {"input": "FEMM - PoW! (Music Video)", "expected": ["FEMM", "PoW!"]},
    # Song with a separator in its name (-), and unparenthesised "official video"
    # https://youtu.be/ti1W7Zu8j9k
    {
        "input": "The Wombats - Anti-D Official Video",
        "expected": ["The Wombats", "Anti-D"],
    },
    # Words containing "…ver" should not be removed--only standalone "ver(.)".
    {"input": "4MINUTE – Whatever", "expected": ["4MINUTE", "Whatever"]},
    {"input": "4MINUTE – Whatever (Test Ver)", "expected": ["4MINUTE", "Whatever"]},
    # Quoted song title
    # https://www.youtube.com/watch?v=VVF0zxw4tuM
    {"input": 'Low Roar - "Half Asleep"', "expected": ["Low Roar", "Half Asleep"]},
    # Quoted song title _and_ "official video"
    # https://www.youtube.com/watch?v=qsWl1--Niyg
    {
        "input": "4MINUTE - 'Volume Up' (Official Music Video)",
        "expected": ["4MINUTE", "Volume Up"],
    },
    # Things with punctuation in front
    # https://www.youtube.com/watch?v=zsF5y1XhGuA
    {
        "input": "...AND YOU WILL KNOW US BY THE TRAIL OF DEAD - Summer Of All Dead Souls",
        "expected": [
            "...AND YOU WILL KNOW US BY THE TRAIL OF DEAD",
            "Summer Of All Dead Souls",
        ],
    },
    # File extensions _and_ "official video"
    # https://www.youtube.com/watch?v=ZPjwdiD24Kg
    {
        "input": "Low Roar - Give Up (Official Video).mov",
        "expected": ["Low Roar", "Give Up"],
    },
    # A separator with _only_ fluff like "MV" on one side
    # https://www.youtube.com/watch?v=yRMvzyN-__Q
    {
        "input": "MV_Planet Shiver_Rainbow [feat. Crush]",
        "expected": ["Planet Shiver", "Rainbow"],
    },
    # "Official MV"
    # https://www.youtube.com/watch?v=qSKPj--tyiM
    {
        "input": "임정희 Lim Jeong Hee - I.O.U Official MV",
        "expected": ["임정희 Lim Jeong Hee", "I.O.U"],
    },
    # 4K, see https://github.com/goto-bus-stop/get-artist-title/issues/20
    {
        "input": "Big Limit [@RealBigLimit] - Samurai Jack [Music Video] (4K)",
        "expected": ["Big Limit", "Samurai Jack"],
    },
    # sizes like 720p
    {
        "input": "Big Limit [@RealBigLimit] - Samurai Jack [Music Video] (720p)",
        "expected": ["Big Limit", "Samurai Jack"],
    },
]


class TestSequence(with_metaclass(MetaTestSequence, unittest.TestCase)):
    test_cases = tests
    test_type = __file__


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Remove common fluff
"""
import re


def clean_common_fluff(title):
    """
    Clean common fluff from title
    """
    # Sub Pop includes "(not the video)" on audio tracks.
    # The " video" part might be stripped by other plugins.
    title = re.sub(r"\(not the( video)?\)\s*$", "", title)
    # Lyrics videos
    title = re.sub(
        r"(\s*[-~_/]\s*)?\b(with\s+)?lyrics\s*", "", title, flags=re.IGNORECASE
    )
    title = re.sub(r"\(\s*(with\s+)?lyrics\s*\)\s*", "", title, flags=re.IGNORECASE)
    title = re.sub(r"\s*\(\s*\)", "", title)  # ()
    return title.strip()
